#!/bin/bash

# This is a live GNU/Linux USB deployment script.
# It writes the attached operating system on the selected block device.
# This script was produced by livemaker.
# http://github.com/Debwin/livemaker

[ "$#" = '1' ] || { echo "Usage: $0 USB_DEVICE"; exit 0; }
[ -b "$1" ] || { echo "Error: '$1' is not a block (USB) device"; exit 1; }
[ $EUID -lt 500 ] || { echo "Error: Need superuser permission to write to device."; exit 2; }
declare -A packages
packages=([parted]=parted [mkfs.ext4]=e2fsprogs [mkfs.ntfs]=ntfs-3g [tar]=tar [sed]=sed [sort]=coreutils [mount]=mount)
for cmd in "${!packages[@]}"; do which $cmd >/dev/null || { echo "Error: missing '$cmd'. Please install package: ${package[$cmd]}"; exit 3; }; done
echo "This will erase all data on '$1' device."
read -p "Do you wish to continue? (y/N) "
[ "$(echo ${REPLY::1} | tr '[:upper:]' '[:lower:]')" = 'y' ] || { echo 'Aborting.'; exit 0; }
set -e
readonly DEVICE="$1"

# partitions from livemaker's build.conf
partitions='
%%PARTITIONS%%
'

partitions="$(echo "$partitions" | nl)"

unmount_partitions() {
  local n
  while read n _ || [ "$n" ]; do
    umount "$DEVICE$n" 2>/dev/null || true
  done < <(echo "$partitions" | sort -k 2 -r)
}

unmount_partitions

# create disklabel and partitions
parted() { command parted --script --align optimal "$@"; }
parted "$DEVICE" mklabel msdos
part_start=1
while read n where fstype size || [ "$where" ]; do
  [ "$where" ] || continue
  [ "${fstype::3}" = 'ext' ] && fstype='ext2'
  [ "$size" != '-' ] && part_end="$(($part_start + $size))MB" || { part_end='100%' size=0; }
  [ "$where" = '/' ] && root_partition="$n"
  parted "$DEVICE" mkpart primary "$fstype" "${part_start}MB" "$part_end"
  part_start=$(($part_start + $size + 1))
done < <(echo "$partitions")
parted "$DEVICE" set "$root_partition" boot on
sync; sync; sleep 1

unmount_partitions

# format partitions
while read n where fstype _ || [ "$where" ]; do
  [ "$where" ] || continue
  [ "$n" = '1' ]         && label='-L live-usb'
  [ "$where" = '/' ]     && label='-L live-rootfs'
  [ "$where" = '/home' ] && label='-L live-home'
  unmount_partitions
  case "$fstype" in
    ntfs) mkfs.ntfs --no-indexing   $label "$DEVICE$n" ;;
    ext*) mkfs.ext4 -O ^has_journal $label "$DEVICE$n" ;;
    fat*) mkfs.vfat ${label/-L/-n} "$DEVICE$n" ;;
    *)    mkfs.$fstype $label "$DEVICE$n" ;;
  esac
  n=$(($n + 1))
done < <(echo "$partitions")

# mount filesystems
CHROOT="$(mktemp -d)"
while read n where fstype _ || [ "$where" ]; do
  [[ "$where" && "$where" != '-' ]] || continue
  case "$fstype" in
    ntfs) fstype='ntfs-3g' ;;
    ext*) fstype='ext4'    ;;
    fat*) fstype='vfat'    ;;
  esac
  mkdir -p "$CHROOT$where"
  mount -t "$fstype" "$DEVICE$n" "$CHROOT$where"
done < <(echo "$partitions" | sort -k 2)

# copy files over
echo 'Copying files. This may take a long time ...'
sed '1,/^#deb153443eb0b8bbd24b5282ae0b7e03489f85e2.*/d' "$0" | tar -xzf - -C "$CHROOT"
#~ cat targz_payload | tar -xzf - -C "$CHROOT"

# fix grub
function chroot_exec { chroot "$CHROOT" /bin/bash -c "$1";}
for fs in dev dev/pts proc sys; do mount -o bind "/$fs" "$CHROOT/$fs"; done
dd if=/dev/zero of="$DEVICE" bs=512 seek=1 count=2047  # fixes later "grub-setup: error: hd0 appears to contain a iso9660 filesystem which isn't known to reserve space for DOS-style boot"
chroot_exec "/usr/sbin/grub-install --modules='part_msdos' --boot-directory=/boot $DEVICE"
chroot_exec "rm /boot/grub/device.map ; /usr/sbin/update-grub ; rm /boot/grub/device.map ; sync"
# TODO run custom scripts
for fs in dev/pts dev proc sys; do umount "$CHROOT/$fs"; done

unmount_partitions
exit 0

# DO NOT CHANGE OR REMOVE BELOW LINE, WHICH MUST END WITH ONE NEW LINE
#deb153443eb0b8bbd24b5282ae0b7e03489f85e2
